/*
Making Czech/Slovak LaTeX2e format under OS/2, requires REXX
(C) 1995, Z. Wagner - Ice Bear Soft

Usage:
  cslatex [base=<base_dir>] [latex=<LaTeX_dir>] [emtexopt=<emTeX_options>]
          [tex=<tex>][csltx=<cs_ltx_dir>] [fmt=<fmt_dir>] [tcp=<code_page>]
          [fmtname=<fmt_name>] [dc=<dc>]
  where
    <base_dir>      = location of the LaTeX2e base, default: dist\latex\base
    <LaTeX_dir>     = location of the resulting files under texinput,
                      default: latex2e
    <cs_ltx_dir>    = location of the source files for CS support,
                      default: dist\cslt2e
    <emTeX_options> = options for running emTeX, default: see below
    <tex>           = version of TeX to run, default: tex386
    <fmt_dir>       = output directory for formats, usually unnecessary
    <code_page>     = TeX code page (must be in data subdirectory),
                      default: 852_il2.tcp
    <fmt_name>      = name of the format input file, default: cslatex.ini
    <dc>            = location of DC font metrics, default: tfm\dc

Environmental variable "emtexdir" must be set and all directories must
reside below it. If variable "emtexopt" is defined in the system, it is used,
otherwise the batch uses its own default which is "/mt65000 /mp65000".
You can specify your own options on the command line. If you want to use
the above written options instead of the options specified in config.sys,
specify emtexopt=default on the command line. The tex program must either
be located in the directory specified by PATH environment variable or you
must enter the whole path in the "tex=" parameter on the command line.
*/

'@echo off'
trace off
call RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
call SysLoadFuncs

/* Some constants */
LastCommentLine=30
PauseAt = 18
CSunpacked = 0

/* Setting defaults */
base = 'dist\latex\base'
latex = 'latex2e'
cslatex = 'cslatex' /* below latex, cannot be changed */
csltx = 'dist\cslt2e'
defaultopt = '/mt65000 /mp65000'
emxopt = ''
tex = 'tex386'
tcp = '852_il2.tcp'
fmtname = 'cslatex.ini'
fmt = ''
dc = 'tfm\dc'
texinput = 'texinput' /* constant, cannot be changed */

/* Remove a backslash from the end of "emtexdir" */
env = 'OS2ENVIRONMENT'
emxd = value('emtexdir',,env)
if emxd = '' then do
  drv = value('emtexdrv',,env)
  if drv = '' then drv = 'C:'
  emxd = drv'\EMTEX'
end
else do
 if substr(emxd,length(emxd)) = ';' then emxd = substr(emxd,length(emxd)-1)
 if substr(emxd,length(emxd)) = '\' then emxd = substr(emxd,length(emxd)-1)
end

/* Get command line arguments */
parse arg arguments
do i = 1 to words(arguments)
  param = word(arguments,i)
  k = pos('=',param)
  if k = 0 then do
    call Syntax
    exit 1
  end

  test = translate(substr(param,1,k-1))

  select
  when test = 'BASE' then base = substr(param,k+1)
  when test = 'LATEX' then latex = substr(param,k+1)
  when test = 'EMTEXOPT' then emxopt = substr(param,k+1)
  when test = 'TEX' then tex = substr(param,k+1)
  when test = 'CSLTX' then csltx = substr(param,k+1)
  when test = 'FMT' then fmt = substr(param,k+1)
  when test = 'FMTNAME' then fmtname = substr(param,k+1)
  when test = 'TCP' then tcp = substr(param,k+1)
  when test = 'DC' then dc = substr(param,k+1)
  otherwise do
    call Syntax
    exit 1
  end
  end
end

/* Build emTeX options */
if translate(emxopt) = 'DEFAULT' then emxopt = defaultopt
else do
  if emxopt = '' then do
    if value('EMTEXOPT',,env) = '' then emxopt = defaultopt
  end
end

/* Build tcp option */
if tcp \= '' then tcp = '/c'||tcp

/* Getting emTeX drive and selecting it */
k = pos(':', emxd)
if k = 2 then do
  drv = substr(emxd,1,2)
  drv
end
else k = 0

/* Changing the dir to emtex */
'cd \'
cdx = 'cd ' substr(emxd,k+1)
cdx

/* Making directories for formats, usually unnecessary */
'md texfmts>nul 2>&1'
'md btexfmts>nul 2>&1'
'md htexfmts>nul 2>&1'

/* Preparing the empty output directories, usually unnecessary */
'md texinput>nul 2>&1'
'cd texinput'
'md' latex'>nul 2>&1'
'cd' latex
'md' cslatex'>nul 2>&1'
cdx

/* Looking for the format directory */

if fmt = '' then do
  if pos('HTEX', translate(tex)) > 0 then call TeXfmt 'HTEXFMT'
  if pos('TEX386', translate(tex)) > 0 | pos('BTEX', translate(tex)) > 0 then call TeXfmt 'BTEXFMT'
  else call TeXfmt 'TEXFMT'
end

/* Preparing TEXINPUT values */

texinp = value('TEXINPUT',, env)
inputs = emxd'\'texinput'\'latex'\cslatex!!;'emxd'\'texinput'\'latex'!!;'
ltxi = emxd'\'texinput'\'latex';'
ltxbase = emxd'\'base';'
csbase = emxd'\'csltx';'
csinput = csbase || ltxi
bases = ltxbase || csbase
texall = inputs || bases || emxd'\'texinput'!!'
ltxinput = ltxi || ltxbase || emxd'\'texinput'!;'

/* Installation loop */

Success = 0
do until hasLtx \= '' | Success

/* Unpacking LaTeX source if necessary */

unpack:
'cd texinput\'latex
hasLtx = stream('latex.ltx', 'C', 'QUERY EXISTS')
if hasLtx \= '' then do
  call stream hasLtx, 'C', 'CLOSE'
end
else do
  'SET TEXINPUT='ltxbase
  tex '/i /8 /o unpack.ins'
  'del unpack.log>nul 2>&1'
end
'cd' cslatex
test = stream('IL2enc.def', 'C', 'QUERY EXISTS')
if test \= '' then do
  CSunpacked = 1
  call stream test, 'C', 'CLOSE'
end
cdx

/* Testing version of the DC fonts */

'cd' dc
hasNewDC = stream('dcr1095.tfm', 'C', 'QUERY EXISTS')
if hasNewDC \= '' then call stream hasNewDC, 'C', 'CLOSE'
cdx

/* Unpacking CS distribution if necessary and if some format is available */

'cd texinput\'latex'\'cslatex
tryfmt = ''
FindFmt = \CSunpacked | hasNewDC = ''
if FindFmt then do
  call SysFileTree fmt'\*.fmt', fmts, 'FO'
  'SET TEXINPUT='csinput
  do i = 1 to fmts.0 until \FindFmt
    tryfmt = StripFMT(fmts.i)
    if \CSunpacked then CSunpacked = CSunpack(tryfmt)
    FindFmt = \CSunpacked
  end
end
cdx

/* Making dummy LaTeX format so that CS distribution may be unpacked */

if FindFmt then do
  /* We need some hyphen.tex, hyphen.en is in CsTeX */
  'cd' fmt
  hyphen = stream('hyphen.tex', 'C', 'QUERY EXISTS')
  if hyphen = '' then do
    call lineout 'hyphen.tex', '\language=0'
    call lineout 'hyphen.tex', '\patterns{.asi1}'
    call lineout 'hyphen.tex', '\endinput'
  end
  call stream 'hyphen.tex', 'C', 'CLOSE'
  'SET TEXINPUT='ltxinput
  if \Make() then do
    'del hyphen.tex>nul 2>&1'
    say 'Sorry, I cannot make LaTeX2e format needed for unpacking the CS distribution'
    'set TEXINPUT='texinp
    exit 1
  end
  'del hyphen.tex>nul 2>&1'
  parse arg fmtname tryfmt '.' rest
  cdx
end

/* Unpack the CS distribution with LaTeX2e */

'SET TEXINPUT='csinput
if \CSunpacked then CSunpacked = CSunpack(tryfmt)
if \CSunpacked then do
    say 'Sorry, I cannot unpack CS distribution!'
    'set TEXINPUT='texinp
    exit 1
end

/* Unpacking fd files for old dc-fonts */

cdx
'cd' dc
if hasNewDC = '' then do
  cdx
  'cd texinput\'latex
  say 'Generating fd for old dc-fonts'
  'SET TEXINPUT='ltxbase || csbase
  tex '^&'tryfmt csolddc.ins
  if rc = 0 then 'del' csolddc.log
  else do
    say 'fd for old-dc fonts not unpacked, T1 will not work properly'
    'pause'
  end
end
cdx

/* Make the format */

'cd' fmt
'SET TEXINPUT='texall
Success = Make()
cdx

/* End of the loop */

end

/* Test the result */

if \Success then do
  say 'Sorry, something is wrong in your system. The format was not created.'
  say 'Try to delete some hyphenation from cshyphen.cfg and run cslatex again.'
  'set TEXINPUT='texinp
  exit 1
end

say 'The CS LaTeX2e format has been successfully created.'
'set TEXINPUT='texinp
exit 0

/* Procedures */

/* Syntax */
Syntax: procedure expose LastCommentLine PauseAt
do i = 2 to LastCommentLine
  say sourceline(i)
  if i = PauseAt then 'pause'
end
return

/* Getting format directory */
TeXfmt: procedure expose env fmt emxd
parse arg arg
fmt = value(arg,,env)
if fmt = '' then fmt = emxd'\'arg'S'
return

/* Unpacking CS distribution with a specified format, returns zero on failure */

CSunpack: procedure expose tex latex cslatex cdx
parse arg fmtnam
cdx
'cd texinput\'latex'\'cslatex
tex '^&'fmtnam 'csunpack.ins'
rrc = rc
if rrc \= 0 then return 0
'del *.log>nul 2>&1'
return 1

/* Stripping path and extension from the format file */
StripFMT: procedure
parse arg arg
k = pos('\', reverse(arg))
n = length(arg)
return substr(arg, n-k+2, k-5)

/* Making the CS-LaTeX format, returns 1 on success, 0 on failure */
Make: procedure expose tex base latex fmt emxopt tcp fmtname
tex '/i /8 /o' tcp emxopt fmtname
rrc = rc
'del texsys.aux>nul 2>&1'
if rrc \= 0 then return 0
return 1
